package CBAIS::Actions::Cchmod;

=head1 NAME

CBAIS::Actions::Cchmod - change file permissions and mode

=head1 SYNOPSIS

$cchmod = new CBAIS::Actions::Cchmod(FILE   => '/etc/{passwd,group}',
                                     MODE  => '0644',
                                     OWNER => 'root',
                                     GROUP => 'root',
                                );

if ($cchmod->execute())
{
    print "cchmod /etc/{passwd,group}\n";
}

if ($cchmod->verify())
{
    print "/etc/{passwd,group} pems and ownweship are up to date\n";
}

=head1 DESCRIPTION


=head1 SEE ALSO

L<CBAIS::Actions::Action>

L<CBAIS::Actions::MOGAction>

L<perlfunc/glob>

=cut

use Carp;

require Util::Tracer;

require CBAIS::Actions::MOGAction;

@ISA = qw(CBAIS::Actions::MOGAction);

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = CBAIS::Actions::MOGAction->new(@_);

    croak "Missing required key FILE" unless $this->check_keys('FILE');
    $this->setInstallRoot('FILE');
    bless($this, $class);
    return $this;
}

sub check
{
    my ($tracer) = new Util::Tracer(@_);
    return 1;
}

sub execute
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift; 
    my ($file);
    
    $this->{FILES} = [glob($this->{FILE})];
    
    foreach $file (@{$this->{FILES}})
    {
        if (! $this->chown_and_chmod($file))
        {
            $this->{ERROR_MESSAGE} = "chmod of $file failed";
            return 0;
        }
    }
    return 1;
}


sub verify
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;   
    my ($file);

    $this->{FILES} = [glob($this->{FILE})];
   
    foreach $file (@{$this->{FILES}})
    {
        if (! $this->SUPER::verify($file))
        {
            return 0;
        }
    }
    return 1;
}

sub toString
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;
    my ($string);
    if (defined($this->{FILES}))
    {
        $string = 'cchmod ' . join(',',@{$this->{FILES}});
    }
    else
    {
        $string = "cchmod $this->{FILE}";
    }
    return $string . " $this->{OWNER}.$this->{GROUP} $this->{MODE_STR}";
}
