package CBAIS::Actions::Cmd;

=head1 NAME

CBAIS::Actions::Cmd - execute an arbitrary command during CBAIS install

=head1 SYNOPSIS

$cmd = new CBAIS::Actions::Cmd(CMD  => 'cmd_name',
                               WHEN  => 'end',
                               ARGS => ['-l',  '-m',  'val'], 
                               );

if ($cmd->execute())
{
    print "executed ", $cmd->toString(), "\n";
}

=head1 DESCRIPTION

The B<CBAIS::Actions::Cmd> implements a CBAIS Action class which excutes a
command (shell, script, ..) during a host install.


=cut


use Carp;

require Util::Tracer;

use CBAIS::Util::Utils;

require CBAIS::Actions::Action;

@ISA = qw(CBAIS::Actions::Action);

sub new
{
    my ($class) = shift;
    my ($this) = CBAIS::Actions::Action->new(@_);
    croak unless $this->check_keys('CMD');

    if (defined($this->{HOST_SPECIFIC}) &&
        $this->{HOST_SPECIFIC} eq 'yes')
    {
        # add host specifc crap
    }
    if ($this->{CMD} =~ m:^/:)
    {
        # if cmd is specified as an absolute path, append install root to it
        $this->setInstallRoot('CMD');
    }
    bless($this, $class);
    return $this;
}

sub check
{
    my ($this) = shift;
    return 1;
}

sub execute
{
    my ($this) = shift;

    my (@cmd) = ($this->{CMD});
    if (defined($this->{ARGS}))
    {
        push(@cmd, @{$this->{ARGS}});
    }
    my (@retval);
    if (grep(/\$/, @cmd))
    {
        # make it a single string so we get shell argument processing
        @retval = sysCall(join(' ', @cmd));
    }
    else
    {
        @retval = sysCall(@cmd);
    }
    if ( @retvals == 2 )
    {
        $this->{'ERROR_MESSAGE'} =
            join(' ', @cmd) . "failed : $retvals[0], Errno $retvals[1]";
        return 0;
    }
    else
    {
        return 1;
    }
}

sub delayTillEnd
{
    my ($this) = shift;
    return (defined($this->{WHEN}) && ($this->{WHEN} eq 'end'))
}

sub toString()
{
    my ($this) = shift;
    if (defined($this->{ARGS}))
    {
        return "$this->{CMD} " . join(' ', @{$this->{ARGS}});
    }
    else
    {
        return $this->{CMD};
  
    }
}
