package CBAIS::Actions::Link;

=head1 NAME

CBAIS::Actions::Link - create HardLinks

=head1 SYNOPSIS




=head1 DESCRIPTION

The B<CBAIS::Actions::Link> class implements a CBAIS action which creates
links. The verify and undo operations are also supported.

The constructor recognises the following keys

=over 4

=item TARGET

The target file to link to. Must exist and be a file. May be a relative
value (if dir is specified).

=item LINK

The name of the link. May be absolute filename unless the DIR key is specified.

=item DIR

A directory to cd to before creating the link.

=back

=head1 CONFIG FILE EXAMPLES

The following demonstrates a relative and absolute symlink configuration in
a class file.

{
    action => 'link',
    target => '/sbin/ls',
    link   => '/bin/ls',
    
},

=head1 SEE ALSO

L<CBAIS::Actions::Action>

=cut

use Carp;

require Util::Tracer;

use CBAIS::CBAIS;

use base("CBAIS::Actions::Action");

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = new CBAIS::Actions::Link(@_);
    bless($this, $class);
    return $this;
}

sub execute
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;

    return unless $this->_chdir();  

    if (! -f $this->{TARGET})
    {
        $this->{ERROR_MESSAGE} = "$ERROR $this->{TARGET} must be a file";
        return 0;
    }
    if (link($this->{'TARGET'}, $this->{'LINK'}))
    {
        return 1;
    }
    else
    {
        $this->{ERROR_MESSAGE} = "$ERROR Unable to create link :$!";
        return 0;
    }
}

sub verify
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;

    return unless $this->_chdir();

    ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
     $atime,$mtime,$ctime,$blksize,$blocks)
        = stat($this->{LINK});
    if ($nlink < 2)
    {
        $this->{ERROR_MESSAGE} = "$ERROR < 2 ($nlink) links to file $this->{LINK}";
        return 0;
    }
    else
    {
        return 1;
    }
}


sub toString()
{
    my ($this) = shift;
    return "link $this->{'TARGET'} $this->{'LINK'}";

}
