package CBAIS::Actions::Link;
=head1 NAME

CBAIS::Actions::Link - common code for Symlink/HardLink classes

=head1 SYNOPSIS




=head1 DESCRIPTION

The B<CBAIS::Actions::Link> class implements a CBAIS action which creates
links. The verify and undo operations are also supported.

The constructor recognises the following keys

=over 4

=item TARGET

The target file to link to. Must exist and be a file. May be a relative
value (if dir is specified).

=item LINK

The name of the link. May be absolute filename unless the DIR key is specified.

=item DIR

A directory to cd to before creating the link.

=back

=head1 CONFIG FILE EXAMPLES

The following demonstrates a relative and absolute symlink configuration in
a class file.

{
    action => 'link',
    target => '/sbin/ls',
    link   => '/bin/ls',
    
},

=head1 SEE ALSO

L<CBAIS::Actions::Action>

=cut

use Carp;

require Util::Tracer;

use base("CBAIS::Actions::Action");

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = CBAIS::Actions::Action->new(@_);
    croak "Missing required keys from hash"
        unless $this->check_keys('LINK', 'TARGET');

    my (@install_paths) = ('DIR');

    if ($this->{LINK} =~ m:^/:)
    {
        push(@install_paths, 'LINK');
    }
    if ($this->{TARGET} =~ m:^/:)
    {
        push(@install_paths, 'TARGET');
    }
    $this->setInstallRoot(@install_paths);

    bless($this, $class);
    return $this;
}

sub _chdir
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    if (defined($this->{DIR}))
    {
        if (! chdir($this->{DIR}))
        {
            $this->{ERROR_MESSAGE} = "Unable to cd $this->{DIR} : $!";
            return 0;
        }
        $tracer->print("cd $this->{DIR} succeeded");
    }
    return 1;
}

sub check
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    return 1;
}

sub toString
{
    my ($this) = shift;
    if (defined($this->{DIR}))
    {
        return "$this->{ACTION} $this->{DIR}/$this->{LINK} -> $this->{TARGET}";
    }
    else
    {
        return "$this->{ACTION} $this->{LINK} -> $this->{TARGET}"; 
    }
}
