package CBAIS::Actions::Rm;

=head1 NAME

CBAIS::Actions::Rm - remove files and directories

=head1 SYNOPSIS

$rm =
    new CBAIS::Actions::Rm(FILE=> '/usr/bin/yp*', );

if ($rm->execute())
{
    print "removed /usr/bin/yp*\n";
}


=head1 DESCRIPTION

The B<CBAIS::Actions::Rm> class implements a CBAIS action which removes
files. The specified file may contain standard unix filename expansions which
will be procesed using the perl glob function. If the specified file is a
directory, it will be removed (along with all of its contents).

The verify operation is supported. The undo operation is not currently
supported. A future release of CBAIS may support the undo method.

The constructor recognises the following keys

=over 4

=item FILE

The file or directory to remove. May contain shell expansions.

=back

=head1 SEE ALSO

L<CBAIS::Actions::Action>

L<perlfunc/glob>

=cut

use Carp;

use File::Path;

require Util::Tracer;

use CBAIS::CBAIS;

require CBAIS::Actions::Action;

our @ISA = qw(CBAIS::Actions::Action);

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift; 
    my ($this) = CBAIS::Actions::Action->new(@_);
    
    croak "Missing required key FILE" unless $this->check_keys('FILE');
    $this->setInstallRoot('FILE');
    bless($this, $class);
    return $this;
}

sub check
{
    my ($tracer) = new Util::Tracer(@_);
    return 1;
}

sub execute
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift; 
    my ($file);
    my ($removed) = 0;
    
    $this->{FILES} = [glob($this->{FILE})];
    
    foreach $file (@{$this->{FILES}})
    {
        if (-d  $file)
        {
            my ($retval);
            if (! ($retval = rmtree($file)))
            {
                $this->{ERROR_MESSAGE} = "unable to remove $file";
                return 0;
            }
            $removed += $retval;
        }
        else
        {
            if (! unlink($file))
            {
                $this->{ERROR_MESSAGE} = "unable to remove $file";
                return 0;  
            }
            $removed++;
        }
        $tracer->print("removed $file");
    }
    $tracer->print("removed  $removed files/directories");
    return 1;
    

    return 1;
}

sub verify
{
   my ($tracer) = new Util::Tracer(@_); 
   my ($this) = shift;
   my (@files) = glob($this->{FILE});

   $tracer->print("Found files " . join(',', @files));
   if (@files == 0)
   {
       return 1;
   }
   else
   {
       $this->{ERROR_MESSAGE} = $ERROR . join(',', @files) . ' not removed';
       return 0;
   }
   
}
sub toString
{
  my ($tracer) = new Util::Tracer(@_); 
  my ($this) = shift;
  my ($string);
  if (defined($this->{FILES}))
  {
      return 'rm ' . join(',',@{$this->{FILES}});
  }
  else
  {
      return "rm $this->{FILE}";
  }
}
