package CBAIS::Config::CBAIS;

=head1 NAME

B<CBAIS::Config::CBAIS> - provide access to CBAIS global configs.

=head2 DESCRIPTION

This class provides access to CBAIS configs. This hides the format of
the configuration files from the CBAIS application. 

=head1 METHODS


=cut

# require CBAIS.conf;



require Exporter;

use Carp;

use Util::Tracer;

our @ISA = qw(Exporter);

our @EXPORT = qw(getCBAISConfig);

my ($singleton) = 0;;

sub getCBAISConfig()
{
    if (! $singleton)
    {
        $singleton = new CBAIS::Config::CBAIS;
    }
    return  $singleton;
}

sub new
{
    my ($tracer) = new Util::Tracer(@_);

    my ($class) = shift;
    # later on ....
    # FIXME
    # if XML then loadXML CBAISConf.xml
    my ($conf_file) = 'CBAIS.conf';
    $conf_file = $ENV{CBAIS_CONF} if defined $ENV{CBAIS_CONF};
    eval "require \'$conf_file\'";
    if ($@)
    {
        croak "Unable to load CBAIS.conf : $@";
    }

    my ($this) = {};
    bless ($this, $class);
    return $this;
}


=head2 getActions()

return a list of valid actions.

=cut

sub getActions
{
    return (keys %{$CBAIS::Actions});
}

=head2 getActionPackage() action

This method returns the package (class) to instansiate for the given action.

=cut

sub getActionPackage
{    
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($action) = shift;
    if (! defined($CBAIS::Actions))
    {
        # FIXME - warning
        die  "Mising Action config section from CBAIS.conf!\n"; 
    }
    if (exists($CBAIS::Actions->{$action}->{package}))
    {
        return $CBAIS::Actions->{$action}->{package};
    }
    else
    {
        return 0;
    }
}

=head2 getActionFilters() action

This method returns a list of filters to apply to an action configuration. It
allows action configurations to be altered before being passed to action
object constructors. It may return the empty list.

=cut

sub getActionFilters
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($action) = shift;   

    if (! defined($CBAIS::Actions))
    {
        # FIXME - warning
        die  "Mising Action config section from CBAIS.conf!\n"; 
    }
    if (exists($CBAIS::Actions->{$action}->{filters}))
    {
        return @{$CBAIS::Actions->{$action}->{filters}};
    }
    else
    {
        $tracer->print("No filters defined for $action");
        return ();
    }
}

# returns an Options section item if defined , else the default (if passed)
# or 0.

sub getOption
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($option) = shift;
    my ($default) = 0;
    if (@_)
    {
        $default = shift;
    }

    if (! defined($CBAIS::Options->{$option}))
    {
        $default ? return $default : return 0;
    }
    else
    {
        return $CBAIS::Options->{$option};
    }
}

#
sub getProjectRoot
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this, $project) = @_;

    if (! (defined($CBAIS::Projects->{$project}) &&
           defined($CBAIS::Projects->{$project}->{root})))
    {
        return 0;
    }
    else
    {
        return $CBAIS::Projects->{$project}->{root};
    }
}
