package CBAIS::Config::Class;

# model a Class file config

use Data::Dumper;

use CBAIS::Util::Utils;

use CBAIS::Config::CBAIS;
use CBAIS::Config::UIDFilter;


sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = {};
    $this->{FILE_LOCATOR} = shift;
    $this->{CBAIS_CONFIG} = getCBAISConfig();

    bless($this, $class);

    if (@_)
    {
        $this->{CLASS} = shift;
    }
    

    return $this;
}

#
#
sub loadClassFile
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($specifiers);
    my ($locator) = $this->{FILE_LOCATOR};
    my ($filename);
    my ($file);
    
    if (@_)
    {
        $this->{CLASS} = shift;  
    }
    $specifiers =
        $this->{CBAIS_CONFIG}->getOption('ClassFileSpecifiers', 'None');
                                                   
    $filename = "$this->{CLASS}.cf";
    $tracer->print("Using specifier $specifiers to locate class file");
    if ($specifiers eq 'None')
    {
        # just load out of conf dir
    }
    elsif ($specifiers eq 'Arch')
    {
        $file = $locator->getSpecificFile('conf/Classes', $filename);
    }
    elsif ($specifiers eq 'Host')
    {
        $file = $locator->getHostSpecificFile('conf/Classes', $filename);
    }
    elsif ($specifiers eq 'Both')
    {
        $file = $locator->getHostArchSpecificFile('conf/Classes', $filename);
    }
    else
    {
        die "Invalid setting ($specifiers) for ClassFileSpecifiers in CBAIS config!";
    }
    if (! $file)
    {
        die "Unable to load $filename!";
    }
    else
    {
        my ($code) = "require \'$file\'";
        $tracer->print("About to eval; :$code:");
        $this->{ACTIONS} = eval $code;
        if ($@)
        {
            $tracer->print("Error loading config $filename: $@");
        }
        else
        {
            my ($action);
            my ($no_actions) = 0;
            my (@actions);
            foreach $action (@{$this->{ACTIONS}})
            {
                $no_actions++;
                $action = ucHash($action);
                $action->{CLASS} = $this->{CLASS}; 

                my (@filters) =
                    $this->{CBAIS_CONFIG}->getActionFilters($action->{ACTION});
                if (@filters)
                {
                    my $filter;
                    foreach $filter (@filters)
                    {
                        $filter->filter($action);
                        $tracer->print("applied filter $filter to " .
                                       Dumper($action));
                    }
                }
            }
            
            $tracer->print("Loaded $file ($no_actions actions)");
            $tracer->print('Added class name to actions');
            return 1;
        }
    }
}

sub getActions()
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift; 

    
    if (! defined ($this->{ACTIONS}))
    {
        return 0;
    }
    else
    {
        if (@_)
        {
            my ($action);
            my (@actions);
            foreach $action (@{$this->{ACTIONS}})
            {
                if (grep(/^$action->{ACTION}$/, @_))
                {
                    $tracer->print("Matched action $action->{ACTION}");
                    push(@actions, $action);
                }
                else
                {
                    $tracer->print("Bypassing action " .
                                   Dumper($action));
                }
            }
            return @actions;
        }
        else
        {
            return @{$this->{ACTIONS}};
        }
    }
}

1;
