package CBAIS::Host;

use Data::Dumper;

use CBAIS::CBAIS;

require CBAIS::Actions::Action;
require CBAIS::Actions::Append;
require CBAIS::Actions::Cmd;
require CBAIS::Actions::Cchmod;
require CBAIS::Actions::Mkdir;
require CBAIS::Actions::Copy;
require CBAIS::Actions::Pkg;
require CBAIS::Actions::Rm;

# this should change .. but for now we will only include of this var
if ($^O eq 'linux' || defined($ENV{CBAIS_ENABLE_RPM}))
{
    require CBAIS::Actions::Rpm;
}
require CBAIS::Actions::Symlink;
require CBAIS::Actions::TextAction;
require CBAIS::Actions::FileHolder;

# require CBAIS::Config::Class;
# require CBAIS::Path::FileLocator;

# require CBAIS::Actions::ActionFactory;

# model a host
# can pass in ACTIONS => [ list, of , actions, ]
# reboot command (default reboot)
# spool dir      (default /etc/cbais)
#
sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;

    my ($this)  = {
        REBOOT_CMD => 'reboot',
        SPOOL_DIR  => '/etc/cbais',
        @_
        };

    bless($this, $class);
    $tracer->print("Host after initialising =" . Dumper($this));
    return $this;
}

# installs all classes for this host

# [ classes, actions]

sub setActions
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    $this->{ACTIONS} = [@_]; 
}

sub _check
{
    my ($action) = shift;
    if (! $action->check())
    {
        print STDERR $ERROR, $action->toString(), ': ',
        $action->getErrorMessage(), "\n";
        return 0;
    }
    return 1;
}

sub check
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($class, $action);

    $tracer->print('Number actions =' . @{$this->{ACTIONS}});
    ACTION : foreach $action (@{$this->{ACTIONS}})
    {
        if ($action->check())
        {
            print $action->toString(), " OK\n";
        }
        else
        {
            print STDERR $action->getErrorMessage(), "\n";
        }
    }
}

sub execute
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($class, $action);
    my ($done) = 0;
    my ($spool) = $this->{SPOOL_DIR};
    
    $tracer->print('Number actions =' . @{$this->{ACTIONS}});
    ACTION : foreach $action (@{$this->{ACTIONS}})
    {
        # $tracer->print("processing " . Dumper($action));
        next ACTION unless _check($action);
        if ($action->isa('CBAIS::Actions::Reboot'))
        {
            if ($this->dump($spool, $done+1))
            {
                print "dumped to $spool\n";
                print "about to do a system $this->{REBOOT_CMD}\n";
                exec $this->{REBOOT_CMD};
            }
            else
            {
                print STDERR "Unable to dump config to $spool\n";
                exit 2;
            }
        }
        if ($action->execute())
        {
            if ($action->isa('CBAIS::Actions::TextAction'))
            {
                print 'Processing ', $action->toString(), "\n";
            }
            else
            {
                print $action->toString(), " succeeded\n";
            }
        }
        else
        {
            print STDERR $action->getErrorMessage(), "\n";
        }
        $done++;
    }
}

sub verify
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    
    ACTION : foreach $action (@{$this->{ACTIONS}})
    {
        if ($action->isa("CBAIS::Actions::TextAction"))
        {
            print $action->toString(), "\n";
            next ACTION;
        }
        next ACTION unless _check($action);
        if ($action->verify())
        {
            print $action->toString(), " OK\n";
        }
        else
        {
            # print STDERR $action->getErrorMessage(), "\n";
            print STDERR $action->getErrorMessage(), "\n";
        }
    }
}

sub undo
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    
    ACTION : foreach $action (@{$this->{ACTIONS}})
    {
        if ($action->isa('CBAIS::Actions::TextAction'))
        {
            print $action->toString(), "\n";
            next ACTION;
        }

        next ACTION unless _check($action);

        if ($action->undo())
        {
            print 'undo ', $action->toString(), " succeeded\n";
        }
        else
        {
            print STDERR $action->getErrorMessage(), "\n";
        }
    }
}

sub show
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    
    ACTION : foreach $action (@{$this->{ACTIONS}})
    {
        next ACTION unless _check($action);
        print $action->toString(), "\n";
    }
}


sub read
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this, $dbase) = @_;
    my (@action_files);
    my ($file, $action);
    
    $dbase = '/tmp/cbais' unless $dbase;

    if (! -d $dbase)
    {
        print STDERR $ERROR, "dir $dbase does not exist!\n";
        return 0;
    }
    @action_files = glob("$dbase/cbais-*.sa");

    foreach $file (@action_files)
    {
        my (@content);
        if (! open (ACTION, "<$file"))
        {
            print STDERR $ERROR, "unable to open $file : $!\n";
            next;
        }
        @content = <ACTION>;
        close ACTION;
        $action = CBAIS::Actions::Action::deserialise(join('', @content));
        $tracer->print("loaded " . $action->toString() . " from $file");
        push(@{$this->{ACTIONS}}, $action);
    }
}


sub send
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this, $command) = @_;

    print "\n<CBAIS_COMMAND>$command</CBAIS_COMMAND>\n";
    ACTION : foreach $action (@{$this->{ACTIONS}})
    {
        
        my ($string) = $action->serialiseData();
        if (! $string)
        {
            print STDERR $ERROR, $action->getErrorMessage(), "\n";
            next ACTION;
        }
        # print STDERR "Sending: <CBAIS_ACTION>\n$string\n</CBAIS_ACTION>\n";
        print "<CBAIS_ACTION>\n$string\n</CBAIS_ACTION>\n";
    }
    return 1;
}

sub receive
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    local (*INPUT) = shift;
    my ($line);
    my ($command) = "undefined_command";
    my ($action_string);
    my ($action);
    my ($loading) = 0;
    
    while ($line = <INPUT>)
    {
        # print STDERR "Read line $line\n";
        next if $line =~ /^$/;
        if ($line =~ m/^<CBAIS_COMMAND>/)
        {
            $command = $line;
            $command =~ s/<CBAIS_COMMAND>//;
            $command =~ s:</CBAIS_COMMAND>::;
            $tracer->print("read command $command");
            next;
        }
        elsif ($line =~ m/^<CBAIS_ACTION>/)
        {
            $loading = 1;
            $action_string = "";
            next;
        }
        elsif ($line =~ m:^</CBAIS_ACTION>:)
        {
            if ($action_string)
            {
                $action = CBAIS::Actions::Action::deserialise($action_string);
                $tracer->print("read " . $action->toString() . "from stdin");
                push(@{$this->{ACTIONS}}, $action);
            }
            $loading = 0;
            $action_string = "";
        }
        elsif ($loading)
        {
            $action_string .= $line;
        }
    }
    $tracer->print("Read $#{$this->{ACTIONS}} actions");
    return $command;
}


# file
# file from
# file from to
sub dump
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($dbase);
    
    my ($from, $to) = (0, $#{$this->{ACTIONS}});
    
    if (@_)
    {
        $dbase = shift;   
    }
    else
    {
        $dbase = '/tmp/cbais';
    }
    
    if (@_)
    {
        $from =  shift;
    }

    if (@_)
    {
        $to = shift; 
    }
    


    if (! -d $dbase)
    {
        # dump each action in individual files
        if (! mkdir $dbase)
        {
            print STDERR "$ERROR unable to mkdir $dbase : $!\n";
            return 0;
        }
    }
    else
    {
        # remove any existing .sa files
        my (@files) = glob("$dbase/cbais-*.sa");
        if (@files)
        {
            $tracer->print("removing existing .sa files (" .
                           join(', ', @files), ")");
            unlink @files;
        }
    }
    
    my ($i) = 0;
    my ($filename);
    # ACTION : foreach $action (@{$this->{ACTIONS}})
    $tracer->print("dumping actions $from .. $to");
    ACTION : for ($i = $from; $i < $to; $i++)
    {
        $action = $this->{ACTIONS}->[$i];
        $filename = sprintf "cbais-%05u", $i;
        $filename = "$dbase/$filename.sa";
        if (! open(STORE,">$filename"))
        {
            print STDERR "$ERROR unable to open $filename : $!\n";
            return 0;
        }
        my ($string) = $action->serialise();
        if (! $string)
        {
            print STDERR $ERROR, $action->getErrorMessage(), "\n";
            next ACTION;
        }
        $tracer->print("writing ($filename) $string");
        print STORE $string;
        if (! close(STORE))
        {
            print STDERR "$ERROR unable to close $filename : $!\n";
            return 0;
        }
    }
    return 1;
}
1;
