package CBAIS::Util::FileHolder;

=head1 NAME

B<CBAIS::Util::File> - handle file storage issues

=head1 DESCRIPTION


=cut


use MIME::Base64;

use Data::Dumper;

sub new
{
    my ($tracer) = new Util::Tracer(@_);
    my ($class) = shift;
    my ($this) = {
        @_,
        ENCODED => 0,
    };
    
    bless($this, $class);
    $this->{FILE} = "unset";
    return $this;
}


sub setFile
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    $this->{FILE} = shift;
}

sub isLoaded()
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    $tracer->print(Dumper($this));
    return (defined($this->{TEXT_DATA}) || defined($this->{BINARY_DATA}));
}


sub load
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    my ($file);
    my (@file);
    my ($offset) = 0;

    
    if (! -f $this->{FILE})
    {
        $this->{ERROR_MESSAGE} = "File  $this->{FILE} does not exist";
        return 0;
    }

    if (! open(FILE, "<$this->{FILE}"))
    {
        $this->{ERROR_MESSAGE} = "Unable to open $this->{FILE} : $!";
        return 0;
    }
    
    if (-T $this->{FILE})
    {
        @{$this->{TEXT_DATA}} = <FILE>;
        close(FILE);
        $tracer->print("read $this->{FILE} ($#{$this->{TEXT_DATA}} lines)");
    }
    else
    {
        my ($line);
        $tracer->print("reading binary file");
        while ($status = read(FILE, $file, 16384, $offset))
        {
            $offset += $status;
        }
        $tracer->print("read $this->{FILE} ($offset bytes)");
        close(FILE);
        $this->{BINARY_DATA} = $file;
    }
    return 1;
}

sub getErrorMessage
{
    my ($this) = shift;
    return $this->{ERROR_MESSAGE};
}

sub encode
{
    my ($tracer) = new Util::Tracer(@_);
    my ($this) = shift;
    if (defined($this->{BINARY_DATA}) && (! $this->{ENCODED}))
    {
        $this->{BINARY_DATA} = encode_base64($this->{BINARY_DATA});
        $this->{ENCODED} = 1;
    }
}

sub decode
{
    my ($this) = shift;
    return 0 unless $this->{ENCODED};
    if (defined($this->{BINARY_DATA}) && $this->{ENCODED})
    {
        $this->{BINARY_DATA} = decode_base64($this->{BINARY_DATA});
        $this->{ENCODED} = 0;
    }
}

sub write
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this, $filename) = @_;
    my ($line);

    $this->decode() if $this->isBinary();
    
    $tracer->print("Writing $filename");
    
    if (! open(FILE, ">$filename"))
    {
        $this->{ERROR_MESSAGE} = "Unable to write to $filename : $!";
        return 0;
    }
    if (defined($this->{TEXT_DATA}))
    {
        foreach $line (@{$this->{TEXT_DATA}})
        {
            print FILE $line;
        }
    }
    elsif (defined($this->{BINARY_DATA}))
    {
        my ($length) = length($this->{BINARY_DATA});
        my ($written) = 0;
        my ($status);
        while (($written < $length) &&
               ($status =
                syswrite(FILE, $this->{BINARY_DATA}, 16384, $written)))
        {
            $written += $status;
        } 
    }
    
    if (! close(FILE))
    {
        $this->{ERROR_MESSAGE} = "Unable to close $filename : $!";
        return 0; 
    }
    return 1;
}

sub getAsArray
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this, $filename) = shift; 

    if (defined($this->{TEXT_DATA}))
    {
        return @{$this->{TEXT_DATA}};
    }
    return ();
}

sub getAsScalar
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift; 

    if (defined($this->{BINARY_DATA}))
    {
        return $this->{BINARY_DATA};
    }
    elsif (defined($this->{TEXT_DATA}))
    {
        return join('', @{$this->{TEXT_DATA}});
    }
    return 0;
}

sub isBinary
{
    my ($tracer) = new Util::Tracer(@_); 
    my ($this) = shift;

    # print "Inside isBinary - textdata= $this->{TEXT_DATA}\n";
    return 1 if defined $this->{BINARY_DATA};
    return 0 if defined $this->{TEXT_DATA};
    return 0 if -T $this->{FILE};
    return 1;
}
1;
