package CBAIS::Util::Project;

=head1 NAME

B<CBAIS::Util::Project> - model a CBAIS project tree

=head1 SYNOPSIS

=cut

use CBAIS::Config::CBAIS;

use Util::Tracer;


# singleton
my ($projects) = {};

sub getProject
{
    my ($tracer) = new Util::Tracer(@_);
    my ($project) = shift;

    if (! defined($projects->{$project}))
    {
        $tracer->print("Created project $project");
        $projects->{$project} = new CBAIS::Util::Project($project);
    }
    return $projects->{$project};
}
    
sub new
{
#    my ($tracer) = new system::debug::tracer(@_);
    my ($tracer) = new Util::Tracer(@_);

    my ($class, $project) = @_;
    my ($project_root);

    #
    # FIXME - add logic to do more for a project and warn of non configured
    # projects 
    if (defined($CBAIS::Projects->{$project}->{root}))
    {
        $project_root = $CBAIS::Projects->{$project}->{root};
        $tracer->print("project root (from config file) = $project_root");
    }
    elsif (defined($ENV{CBAIS_ROOT}))
    {
        $project_root = "$ENV{CBAIS_ROOT}/$project";
        $tracer->print("project root (from env var) = $project_root");
    }
    else
    {
        $project_root = "./$project";
        $tracer->print("project root (default) = $project_root");
        
    }
    
    my ($this) = {
        PROJECT => $project,
        PROJECT_ROOT => $project_root,
        @_
    };
    bless($this, $class);
}

sub getProjectRoot
{
    my ($this) = shift;
    return $this->{PROJECT_ROOT};
}

sub setProjectRoot
{
    my ($this) = shift;
    $this->{PROJECT_ROOT} = shift;
    if (! -d $this->{PROJECT_ROOT})
    {
        print STDERR "dir $this->{PROJECT_ROOT} does not exist!\n";
    }
}

1;
