package CBAIS::Util::Utils;

=head1 NAME

B<CBAIS::Util::Utils> - miscellaneous utility methods used throughout CBAIS

=head1 DESCRIPTION

=cut

use Data::Dumper;

require Exporter;

use Util::Tracer;

our @ISA = qw(Exporter);

our @EXPORT = qw(majorOSRelease genericArch  parseExcludeList ucHash sysCall);

=head2 majorOSRelease OS Release

process the given OS release and return its major OS release. The release
will be given a .x suffix. e.g. A release of 5.5.1 would become 5.x.

=cut

sub majorOSRelease
{
    my ($release) = shift;
    $release =~ s/\..*$/\.x/;
    return $release;
}

# return generic arch for given arch
sub genericArch
{
    my ($tracer) = new Util::Tracer(@_);
    my ($arch) = shift;
    if ($arch =~ m/sun4/)
    {
        return 'sun4';
    }
    elsif ($arch =~ m/i[4,5,6]86/)
    {
        return 'i386'; 
    }
    else
    {
        return 'unknown';
    }
}

#
# exclude string (!X,Y,Z), full_list
=head2 parseExcludeList exlude_string, LIST

the exclude string is split on a comma, and individual items are matched
against the full list. if mathced, they are rmeoevd form the full list.
Returns the full list minus any items in the exclude string.

=cut

sub parseExcludeList
{
    my ($tracer) = new Util::Tracer(@_);
    my ($excludes) = shift;
    my (@full_list) = @_;
    my ($item);
    my (@excludes);
    my (@items);
    
    $excludes =~  s/!//g;
    @excludes = split(',', $excludes);
    foreach $item (@full_list)
    {
        next if grep (/^$item$/,@excludes); 
        push(@items, $item);   
    }
    return (@items);
}

# utility method to upper case hash keys
# assumes single level of nesting
sub ucHash
{
    my ($config) = shift;
    my ($uc_config);
    my ($key);

    foreach $key (keys %{$config})
    {
        $uc_config->{uc($key)} = $config->{$key};
    }
    return $uc_config;
}

=item sysCall (list)

Returns a list with last element set to 0 if success
else an error message and last element set to other than 0.

=cut


sub sysCall
{
    my ($tracer) = new Util::Tracer(@_);
    my (@cmd) = @_;
    my ($retval) = 0xffff & system(@cmd);
    $tracer->print("system(@cmd) returned $retval");
    if ($retval == 0)
    {
        return 0;
    }
    elsif ($retval == 0xff00)
    {
        return ("@cmd _failed_ : $!:", -1);
    }
    elsif ($retval > 0x80)
    {
        $retval >>= 8;
        return ("@cmd ran with non zero exit status : $retval", $retval);

    }
    else
    {
        if ($retval & 0x80)
        {
            $retval &= ~0x80;
            return ("Core dumped from signal $retval", $retval);
        }
        return ("Got Signal $retval", $retval);
    }
}

1;
